package common.util;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

public class CsvUtil {

	private final static String COMMENT_MARK = "#";

	/**
	 * CSVファイル読み込み
	 *
	 * @param fpath ファイルパス
	 * @param encode エンコード
	 * @throws IOException ファイルIO例外
	 * @return 処理結果
	 */
	public static List<ArrayList> readCsv(final String fpath, final String encode)
			throws IOException {
		return readCsv(fpath, encode, ",", false);

	}

	/**
	 *
	 * @param fpath ファイルパス
	 * @param encode エンコード
	 * @param sepa  セパレータ
	 * @param dquoteFlg オンの場合、存在すれば、データ両端のダブルクォートを取り除く
	 * @return 処理結果
	 * @throws IOException  ファイルIO例外
	 */
	public static List<ArrayList> readCsv(
			final String fpath,
			final String encode,
			final String sepa,
			final boolean dquoteFlg)
			throws IOException {

		ArrayList<ArrayList> rsltList = new ArrayList<ArrayList>();

		FileInputStream fis = new FileInputStream(fpath); // CSVデータファイル
		InputStreamReader isr = new InputStreamReader(fis, encode);
		BufferedReader br = new BufferedReader(isr);

		// 最終行まで読み込む
		String line = "";
		while ((line = br.readLine()) != null) {
			//ホワイトスペースのみの行は無視
			line = line.trim();

			int len = line.length();
			//空行は無視
			if (len < 1) {
				continue;
			}

			// 1行をデータの要素に分割
			String[] value = line.split(sepa, -1);

			ArrayList<String> columnList = new ArrayList();
			for (int i = 0; i < value.length; i++) {
				String buff = value[i];
				if (buff.length() > 0) {
					if (dquoteFlg) {
						if (buff.substring(0, 1).equals("\"")
								&& buff.substring(buff.length() - 1,
										buff.length()).equals("\"")) {
							// 先頭と末尾のダブルクォートを取り除く
							buff = buff.substring(1, buff.length() - 1);
						}
					}
				}
				// 1行の各要素をタブ区切りで表示
				columnList.add(buff);
			}
			rsltList.add(columnList);
		}
		br.close();
		isr.close();
		fis.close();

		return rsltList;
	}

	/**
	 *
	 * @param fpath
	 * @param encode
	 * @param sepa
	 * @param dquoteFlg ダブルクォートフタグ　true：データは、ダブルクォートで囲む
	 * @param commentFlg true:コメント行あり　　行先頭が、COMMENT_MARK("#")　はコメント行
	 * @return
	 * @throws IOException
	 */
	public static List<ArrayList<String>> readCsv(
			final String fpath,
			final String encode,
			final String sepa,
			final boolean dquoteFlg,
			final boolean commentFlg)
			throws IOException {

		ArrayList<ArrayList<String>> rsltList = new ArrayList<ArrayList<String>>();

		FileInputStream fis = new FileInputStream(fpath); // CSVデータファイル
		InputStreamReader isr = new InputStreamReader(fis, encode);
		BufferedReader br = new BufferedReader(isr);

		// 最終行まで読み込む
		String line = "";
		while ((line = br.readLine()) != null) {
			//ホワイトスペースのみの行は無視
			line = line.trim();

			int len = line.length();
			//空行は無視
			if (len < 1) {
				continue;
			}
			//コメント行は無視
			if (commentFlg && (line.startsWith(COMMENT_MARK))) {
				continue;
			}

			// 1行をデータの要素に分割
			String[] value = line.split(sepa, -1);

			ArrayList<String> columnList = new ArrayList();
			for (int i = 0; i < value.length; i++) {
				String buff = value[i];
				//両端のスペースを除去
				buff = buff.trim();

				if (buff.length() > 0) {
					if (dquoteFlg) {
						if (buff.substring(0, 1).equals("\"")
								&& buff.substring(buff.length() -1,
										buff.length()).equals("\"")) {
							// 先頭と末尾のダブルクォートを取り除く
							buff = buff.substring(1, buff.length() - 1);
						}
					}
				}
				// 1行の各要素をタブ区切りで表示
				columnList.add(buff);
			}
			rsltList.add(columnList);
		}
		br.close();
		isr.close();
		fis.close();

		return rsltList;
	}

	/**
	 * オブジェクト形式のリストで戻り値を返す
	 * CSV書式
	 *  コメント行：行先頭が、"#"
	 *  １行目　ヘッダ
	 *  ２行目　データオブジェクトのフィールド名
	 *
	 * @param dtoClassName
	 * @param fpath			ファイルパス
	 * @param encode		エンコード
	 * @param sepa			区切り文字
	 * @param dquoteFlg		ダブルクォート有効フラグ true:ダブルクォート有効
	 * @param commentFlg	コメント有効フラグ true:コメント行有効
	 * @return
	 * @throws IOException
	 * @throws ClassNotFoundException
	 * @throws IllegalAccessException
	 * @throws InstantiationException
	 * @throws InvocationTargetException
	 * @throws IllegalArgumentException
	 * @throws SecurityException
	 * @throws NoSuchMethodException
	 */
	public static List<Object> readCsvToObj(
			final String dtoname,
			final String fpath,
			final String encode,
			final String sepa,
			final boolean dquoteFlg,
			final boolean commentFlg
			) throws IOException,
				ClassNotFoundException,
				InstantiationException,
				IllegalAccessException,
				NoSuchMethodException,
				SecurityException,
				IllegalArgumentException,
				InvocationTargetException {

		ArrayList<Object> rsltList = new ArrayList<Object>();

		FileInputStream fis = new FileInputStream(fpath); // CSVデータファイル
		InputStreamReader isr = new InputStreamReader(fis, encode);
		BufferedReader br = new BufferedReader(isr);

		// ファイルを読み込む
		String[] colnames = null;
		int lineNo = 0;
		String line = "";
		while ((line = br.readLine()) != null) {
			//ホワイトスペースのみの行は無視
			line = line.trim();

			int len = line.length();
			//空行は無視
			if (len < 1) {
				continue;
			}
			//コメント有効ならば、コメント行は無視
			//　コメント行:行先頭が, COMMENT_MARK
			if (commentFlg && (line.startsWith(COMMENT_MARK))) {
				continue;
			}
			lineNo++;

			if (lineNo == 1) {
				// 何もしない
				continue;
			} else if (lineNo == 2) {
				// フィールド名の1行をデータの要素に分割
				colnames = line.split(sepa, -1);
				//タイトル両端の ダブル・クオーテーションを取り除く
				bothDeleteDQ(colnames);
				continue;
			}

			// 1行をデータの要素に分割
			String[] values = line.split(sepa, -1);

			//データ両端の ダブル・クオーテーションを取り除く
			bothDeleteDQ(values);

			//オブジェクトに値を設定
			Class<?> clazz = Class.forName(dtoname);
			Object dtoobj = clazz.newInstance();

			for (int i = 0; i < values.length; i++) {
				String value = values[i];

				setValue(dtoobj, colnames[i], value);
			}
			rsltList.add(dtoobj);

		}
		br.close();
		isr.close();
		fis.close();

		return rsltList;
	}

	/**
	 * 両端のダブルクｫーテーション削除
	 * @param titles 文字列の配列
	 */
	static void bothDeleteDQ(String[] titles) {
		//タイトル両端の ダブル・クオーテーションを取り除く
		for (int i = 0; i < titles.length; i++) {
			String value = titles[i];
			if (titles[i].length() >= 2) {
				String left = titles[i].substring(0, 1);
				String right = titles[i].substring(titles[i].length() - 1, titles[i].length());

				if (left.equals("\"") && right.equals("\""))
				{
					value = titles[i].substring(1, titles[i].length() - 1);
				}
			}
			titles[i] = value;
		}
	}

	/**
	 * データオブジェクト設定
	 * @param dto
	 * @param colname
	 * @param value
	 * @throws SecurityException
	 * @throws NoSuchMethodException
	 * @throws InvocationTargetException
	 * @throws IllegalArgumentException
	 * @throws IllegalAccessException
	 */
	static void setValue(Object dto, String colname, Object value)
			throws NoSuchMethodException,
			SecurityException,
			IllegalAccessException,
			IllegalArgumentException,
			InvocationTargetException {

		// メソッド名編集
		String methodName = "set" + colname.substring(0, 1).toUpperCase() + colname.substring(1, colname.length());
		{
			Object[] objs = null;
			// メソッドオブジェクト取得
			Method method = dto.getClass().getMethod(methodName, String.class);
			// メソッド実行
			method.invoke(dto, value);
		}
	}


	/**
	 *
	 * @param fpath
	 * @param encode
	 * @param sepa
	 * @param dquoteFlg ダブルクォートフタグ　true：データは、ダブルクォートで囲む
	 * @param commentFlg コメントフラグ　true:コメント行あり　　行先頭が、COMMENT_MARK("#")　はコメント行
	 * @param readTitle タイトル行リードフラグ　true：タイトル行を読む、false：タイトル行を読まない　タイトル行：有効な1行目
	 * @return
	 * @throws IOException
	 */
	public static List<ArrayList<String>> readCsvWithTitle(
			final String fpath,
			final String encode,
			final String sepa,
			final boolean dquoteFlg,
			final boolean commentFlg,
			final boolean readTitle)
			throws IOException {

		ArrayList<ArrayList<String>> rsltList = new ArrayList<ArrayList<String>>();

		FileInputStream fis = new FileInputStream(fpath); // CSVデータファイル
		InputStreamReader isr = new InputStreamReader(fis, encode);
		BufferedReader br = new BufferedReader(isr);

		// 最終行まで読み込む
		int lineNo = 0 ;    //有効な行の番号

		String line = "";
		while ((line = br.readLine()) != null) {
			//ホワイトスペースのみの行は無視
			line = line.trim();

			int len = line.length();
			//空行は無視
			if (len < 1) {
				continue;
			}
			//コメント行は無視
			if (commentFlg && (line.startsWith(COMMENT_MARK))) {
				continue;
			}
			//行番号を更新
			lineNo++;

			//タイトル行（有効な1行目）は無視
			if ((!readTitle) && (lineNo==1)){
				continue ;
			}
			// 1行をデータの要素に分割
			String[] value = line.split(sepa, -1);

			ArrayList<String> columnList = new ArrayList();
			for (int i = 0; i < value.length; i++) {
				String buff = value[i];
				//両端のスペースを除去
				buff = buff.trim();

				if (buff.length() > 0) {
					if (dquoteFlg) {
						if (buff.substring(0, 1).equals("\"")
								&& buff.substring(buff.length() -1,
										buff.length()).equals("\"")) {
							// 先頭と末尾のダブルクォートを取り除く
							buff = buff.substring(1, buff.length() - 1);
						}
					}
				}
				// 1行の各要素をタブ区切りで表示
				columnList.add(buff);
			}
			rsltList.add(columnList);
		}
		br.close();
		isr.close();
		fis.close();

		return rsltList;
	}

}
